/*
 * decode.c
 *
 * Created: 2014-02-07 14:00:02
 *  Author: dsosnowski
 */ 

#define F_CPU 4000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "decode.h"

struct {
	volatile char status;
	volatile char index;
	char active;
	char passive;
	char delay;
	char data[CODE_LENGTH];
} code;

struct  {
	char status;
	char address;
	char code1;
	char code2;
} command;


//---------------------------------------------------------------
void Decode(void)
{
	char index;
	char mask;
	
	if (code.status == STATUS_COMPLETE)
	{
		if ((code.data[CODE_START_INDEX] == SPECIAL_BIT) && (code.data[CODE_L_INDEX] == L_BIT))
		{
			command.status = COMMAND_EMPTY;
			command.address = 0;
			command.code1 = 0;
			command.code2 = 0;
			
			mask = 0x80;
			index = CODE_ADR_INDEX;
			while (index < CODE_CODE1_INDEX)
			{
				if	(code.data[index] == H_BIT)
				command.address |= mask;
				else if
				(code.data[index] != L_BIT)
				command.status = COMMAND_ERROR;
				
				mask >>= 1;
				index++;
			}
			
			mask = 0x10;
			index = CODE_CODE1_INDEX;
			while (index < CODE_CODE2_INDEX)
			{
				if	(code.data[index] == H_BIT)
				command.code1 |= mask;
				else if
				(code.data[index] != L_BIT)
				command.status = COMMAND_ERROR;
				
				mask >>= 1;
				index++;
			}

			mask = 0x10;
			index = CODE_CODE2_INDEX;
			while (index < CODE_L_INDEX)
			{
				if	(code.data[index] == H_BIT)
				command.code2 |= mask;
				else if
				(code.data[index] != L_BIT)
				command.status = COMMAND_ERROR;
				
				mask >>= 1;
				index++;
			}
			
			//------------------------------------
			if (command.status == COMMAND_EMPTY)
			{
				//if (command.address == MODULE_ADDRESS)
				{
					if (command.code1 == command.code2)
					command.status = COMMAND_READY;
				}
			}
		}
	}
}


//--------------------------------------------
void DecodeInt(void)
{
	/*OCR1A = 0xFF;
	TCCR1A = (1<<WGM11 | 1<<WGM10);
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS12);
	TIMSK1 = (1<<TOIE1);
		
	GIMSK |= (1<<PCIE0);
	PCMSK0 |= (1<<PCINT7);	
	*/
	OCR1A = 32;			//0.064ms
	TCCR1A = (1<<WGM11 | 1<<WGM10);
	//TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS11);
	TIMER_START;
	TIMSK1 = (1<<TOIE1);		
	
}


//--------------------------------------------
/*ISR (PCINT0_vect)
{
		char timval;
		char new;

		if (DTA_GET)
		{
			TIMER_STOP;
			TIMER_RESET;
			TIMER_CLEAR;
			TIMER_START;
		}
		else
		{
			TIMER_STOP;
			timval = TIMER_VAL;


			new = NULL_BIT;
			//if (timval <= STATE_L_MAX) new = L_BIT;
			if ((timval >= STATE_L_MIN) && (timval <= STATE_L_MAX)) new = L_BIT;
			else if ((timval >= STATE_H_MIN) && (timval <= STATE_H_MAX)) new = H_BIT;
			else if ((timval >= STATE_SPEC_MIN) && (timval <= STATE_SPEC_MAX)) new = SPECIAL_BIT;
		
			if (code.status < STATUS_COMPLETE)
			{
				
				if (new > NULL_BIT)
				{
					if (new == SPECIAL_BIT)
					{
						code.index = 1;
						code.data[code.index] = new;
						code.status = STATUS_INPROGRESS;
					}
					if (code.index < CODE_LENGTH)
					{
						code.index++;
						code.data[code.index] = new;
						if (code.index == CODE_L_INDEX) code.status = STATUS_COMPLETE;
					}
				}
			}
		}
}*/


//---------------------------------------------------------------
//ISR(TIM0_OVF_vect)
ISR(TIM1_OVF_vect)
{
//----------------------------------------
	char new;
	char state = DTA_GET;

//----------------------------------------
	if (code.delay < 0xFF) code.delay++;

//----------------------------------------
	if (code.status == STATUS_IDLE)			//finding special bit
	{
		if (state)		
		{
			if (code.active < 0xFF) code.active++;
			if (code.active > ACTIVE_THRESHOLD) code.passive = 0;
		}
		else
		{
			if (code.passive < 0xFF) code.passive++;
			if (code.passive > PASSIVE_THRESHOLD) code.active = 0;		
			if (code.passive == PASSIVE_THRESHOLD)
			{
				if ((code.active >= STATE_SPEC_MIN) && (code.active <= STATE_SPEC_MAX))
				{
					//new = SPECIAL_BIT;
					code.index = 2;
					code.data[1] = SPECIAL_BIT;
					code.data[2] = SPECIAL_BIT;
					code.status = STATUS_INPROGRESS;
					code.delay = PASSIVE_THRESHOLD;
				}
				code.active = 0;							
			}
		}				
	}
//----------------------------------------
	else
	{
		if (code.status == STATUS_INPROGRESS)			//frame creating
		{
			if (code.delay >= STATE_PASSIVE_MIN)
			{
				if (state)		
				{
					if (code.active < 0xFF) code.active++;
					code.passive = 0;					
				}
				else
				{
					if (code.passive < 0xFF) code.passive++;
					if (code.passive > PASSIVE_THRESHOLD) code.active = 0;
					if (code.passive == PASSIVE_THRESHOLD)
					{
						new = NULL_BIT;
			
						if ((code.active >= STATE_L_MIN) && (code.active <= STATE_L_MAX)) new = L_BIT;
						else if ((code.active >= STATE_H_MIN) && (code.active <= STATE_H_MAX)) new = H_BIT;
						//else if ((code.active >= STATE_SPEC_MIN) && (code.active <= STATE_SPEC_MAX)) new = SPECIAL_BIT;
			
						code.active = 0;
						code.delay = PASSIVE_THRESHOLD;
			
						if (new > NULL_BIT)
						{
							/*if (new == SPECIAL_BIT)
							{
								code.index = 1;
								code.data[code.index] = new;
								//code.status = STATUS_INPROGRESS;
							}*/
							if (code.index < CODE_LENGTH)
							{
								code.index++;
								code.data[code.index] = new;
								if (code.index == CODE_L_INDEX) code.status = STATUS_COMPLETE;
							}
						}
						else
						{
							code.status = STATUS_IDLE;
							code.passive = 0;
						}
					}
				}										
			}
		}				
	}
}
