/*
 * dualcore_rec.c
 *
 * Created: 2014-02-05 15:44:16
 *  Author: dsosnowski
 */ 

#define F_CPU 4000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "decode.h"
#include "basic.h"

void HardInit(void);
#define DELAY_COMMAND		50

struct {
	char prev_command;
	char delay_command;
	char blink;
}system;

extern struct {
	volatile char status;
	volatile char index;
	char data[CODE_LENGTH];
} code;

extern struct  {
	char status;
	char address;
	char code1;
	char code2;
} command;

extern struct {
	char t_on1;
	char t_on2;
	char action;
	char mode;
}inputs;


int main(void)
{
	
	HardInit();
	_delay_ms(600);
	LEDS_OFF;
	
	DecodeInt();
	sei();
	
    while(1)
    {
		inputs.mode = 0;

 		//------------------------------------------------
 		if (code.status == STATUS_COMPLETE)
 		{
	 		Decode();	
	 		//------------------------------------------------
	 		if (command.status == COMMAND_READY)
	 		{
				if (command.code1 != system.prev_command) 
				{
					if (command.code1 == COMMAND_1) inputs.action = 1;
					if (command.code1 == COMMAND_2) inputs.action = 2;
					_delay_ms(5);	
				}
			}
			system.prev_command = command.code1;												
			system.delay_command = DELAY_COMMAND;									
			command.code1 = 0;
			command.status = COMMAND_EMPTY;
			code.status = STATUS_IDLE;
		}

		//----------------------------------------------------
		if (system.delay_command) 
			system.delay_command--;
		else 
			system.prev_command = 0;

		if (inputs.mode == 1)
		{
			if (system.prev_command)			
			{
				if (system.prev_command == COMMAND_1) inputs.action = 1;
				if (system.prev_command == COMMAND_2) inputs.action = 2;
			}
		}
		
		//----------------------------------------------------
 		InputScan();	
		 
		if (inputs.action) system.blink = 10; 
		 	 		
		Action();
			
		//----------------------------------------------------		
		system.blink++;
		if (system.blink < 10)
		{
			if (OUT1_GET) 
				LED1_OFF;
			else
				LED1_SET;				
			if (OUT2_GET) 
				LED2_SET;
			else
				LED2_OFF;				
		}
		else
		{
			if (OUT1_GET) 
				LED1_SET;
			else
				LED1_OFF;				
			if (OUT2_GET) 
				LED2_SET;
			else
				LED2_OFF;							
		}		

		//----------------------------------------------------
		_delay_ms(10);
    }

}

	
//--------------------------------------------------
void HardInit(void)
{
	PORTB = (0xFF);
	DDRB = 0x00;
	PORTA = (1<<DTA_PIN | 1<<IN2_PIN | 1<<MISO_PIN | 1<<SCK_PIN | 1<<LED1_PIN | 1<<LED2_PIN);
	DDRA = (1<<LED1_PIN | 1<<OUT1_PIN | 1<<LED2_PIN | 1<<OUT2_PIN);
}
