/*
 * dualcore_trx.c
 *
 * Created: 2014-02-10 14:54:56
 *  Author: dsosnowski
 */  

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "defs.h"
#include "ee_mem.h"
#include "wdog.h"



void HardInit(void);
void IrStart(void);
void SwitchScan(void);
void CodeInit(void);
void CodeTransmitt(void);
void BatteryCheck(void);


#define BIT_PERIOD			100

#define NULL_BIT			0
#define H_BIT				1
#define L_BIT				2
#define SPECIAL_BIT			3

#define CODE_LENGTH			32
#define CODE_NULL_INDEX		0
#define CODE_SPECIAL_INDEX	1
#define CODE_START_INDEX	2
#define CODE_SYNCHRO_INDEX	3	
#define CODE_ADR_INDEX		8
#define CODE_CODE1_INDEX	16
#define CODE_CODE2_INDEX	21
#define CODE_L_INDEX		26
#define CODE_ENDNULL_INDEX	27

#define MODULE_ADDRESS		54
#define COMMAND_1			11
#define COMMAND_2			6
#define COMMAND_CLR			0

#define TIME_TO_SLEEP		8
#define POWER_CRITICAL		510

struct {
	char address;
	char command; 
} ip;

struct {
	char button;
	char gosleep;
} panel;

struct {
	volatile char status;
	volatile char index;
	volatile char i;
	char data[CODE_LENGTH];
} code;


int main(void)
{
    HardInit();
	_delay_ms(100);
	LEDS_OFF;
	_delay_ms(100);
	LEDG_SET;
	LEDR_SET;
	_delay_ms(100);
	LEDS_OFF;
	_delay_ms(100);
	LEDG_SET;
	LEDR_SET;
	_delay_ms(100);
	
	sei();
	
	while(1)
    {		

		BatteryCheck();

		SwitchScan();
			
		if (panel.button)
		{			
			CodeTransmitt();
			panel.gosleep = TIME_TO_SLEEP;
			panel.button = 0;
		}
				
		if (panel.gosleep)
		{			
			CodeTransmitt();	
			panel.gosleep--;
		}

		if (panel.gosleep == 0)
		{
			//wait for end
			while (code.status);
			LEDS_OFF;
			
			//sleep;
			PRR = (1<<PRADC | 1<<PRUSI | 1<<PRTIM0 | 1<<PRTIM1);
			MCUCR |= (1<<SE | 1<<SM1);
			sleep_cpu();
			MCUCR &= ~(1<<SE);
		}
				
		_delay_ms(50);
    }
}

//---------------------------------------
void HardInit(void)
{
	PORTB = (1<<RESET_PIN | 1<<SW1_PIN | 1<<SW2_PIN);
	DDRB = (1<<DTA_PIN);
	
	PORTA = (0xFE);
	DDRA = (1<<LEDG_PIN | 1<<LEDR_PIN);		
	
	GIFR = (1<<PCIF1);
	GIMSK = (1<<PCIE1);
	PCMSK1 = (1<<PCINT8 | 1<<PCINT9);
	
	ACSR = (1<<ACD);
	PRR = (1<<PRTIM0 | 1<<PRADC | 1<<PRUSI | 1<<PRTIM1);
}

//---------------------------------------
void IrStart(void)
{
	PRR &= ~(1<<PRTIM1);	
	//start code timer
	OCR1A = BIT_PERIOD;
	TCNT1 = 0;
	GTCCR = 0;
	TIFR1 = (1<<TOV1); 
	TIMSK1 = (1<<TOIE1);
	TCCR1A = (1<<WGM11 | 1<<WGM10);
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS11 | 1<<CS10);
}

//---------------------------------------
ISR (PCINT1_vect)
{
	//wake up
}

//---------------------------------------
ISR (TIM1_OVF_vect)
{
	if (code.index < CODE_LENGTH)
	{
		if (code.index == 0) 
		{
			code.i = 0;
			code.index = 1;
		}		
		
		if (code.i == 0)			//bit pulse
		{
			LEDG_SET;
			if (code.data[code.index] == 0)		
				DTA_OFF;			//null bit
			else
				DTA_SET;									
		}
		else if (code.i == 1)		//bit 0
		{
			LEDG_OFF;	
			if (code.data[code.index] == 2)
			{
				DTA_OFF;
				code.i++;
			}
		}
		else if (code.i == 2)		//bit 1
		{
			if (code.data[code.index] == 1)	DTA_OFF;		
		}
		
		code.i++;
		if (code.i > 3) 
		{
			code.i = 0;		
			code.index++;
			//if (code.index >= CODE_LENGTH)	code.index = 0;
		}			
	}	
	else
	{
		TCCR1B = 0;
		DTA_OFF;	
		code.status = 0;
	}
	return;
}

//---------------------------------------
void SwitchScan(void)
{
	if (GET_SW1 == 0) panel.button |= 1;
	if (GET_SW2 == 0) panel.button |= 2;
}	

//---------------------------------------
void CodeInit(void)
{
	char mask;
	char index;
	
	code.data[0]  = NULL_BIT;
	code.data[CODE_SPECIAL_INDEX] = SPECIAL_BIT;
	code.data[CODE_START_INDEX] = L_BIT;

	index = CODE_SYNCHRO_INDEX;
	while (index < CODE_ADR_INDEX)
	{
		code.data[index] = L_BIT;
		index++;
	}
		
	mask = 0x80;
	index = CODE_ADR_INDEX;
	while (index < CODE_CODE1_INDEX)
	{
		if (ip.address & mask)
			code.data[index] = H_BIT;				
		else
			code.data[index] = L_BIT;				
		index++;
		mask >>= 1;
	}
	
	mask = 0x10;
	index = CODE_CODE1_INDEX;
	while (index < CODE_CODE2_INDEX)
	{
		if (ip.command & mask)
			code.data[index] = H_BIT;				
		else
			code.data[index] = L_BIT;				
		mask >>= 1;
		index++;
	}

	mask = 0x10;
	index = CODE_CODE2_INDEX;
	while (index < CODE_L_INDEX)
	{
		if (ip.command & mask)
			code.data[index] = H_BIT;				
		else
			code.data[index] = L_BIT;				
		mask >>= 1;
		index++;
	}

	code.data[CODE_L_INDEX] = L_BIT;
	code.data[CODE_L_INDEX + 1] = NULL_BIT;
	code.data[CODE_L_INDEX + 2] = NULL_BIT;
}

//-----------------------------------------------------
void CodeTransmitt(void)
{
	if (code.status == 0)
	{
		LEDG_SET;
		ip.command = COMMAND_CLR;
		if (panel.button & SW_1) ip.command = COMMAND_1;
		if (panel.button & SW_2) ip.command = COMMAND_2;
			
		ip.address = MODULE_ADDRESS;
		CodeInit();
		code.status = 1;
		code.index = 0;
		IrStart();
	}
}

//---------------------------------------------------
void BatteryCheck(void)
{
	PRR &= ~(1<<PRADC);
	ADCSRA = (1<<ADEN | 7);
	ADCSRB = 0;
	ADMUX = (1<<REFS1);		//1.1V
	_delay_ms(2);
	ADCSRA |= (1<<ADSC);
	
	while (ADCSRA & (1<<ADSC));
	if (ADC < POWER_CRITICAL) LEDR_SET;
	
	ADCSRA = 0;
	PRR |= (1<<PRADC);	
}